//+------------------------------------------------------------------+
//|                                                           T3.mq4 |
//|                                                           mladen |
//|                                                                  |
//|                                                                  |
//| original T3 developed by Tim Tilson (TASC January 1998)          |
//| period lag modification by Bob Fulks and Alex Matulich 4/2003    |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_chart_window
#property indicator_buffers 1
#property indicator_color1  Lime
#property indicator_width1  2

//
//
//
//
//

extern string TimeFrame   = "Current time frame";
extern int    T3Period    = 14;
extern int    T3Price     = PRICE_CLOSE;
extern double T3Hot       = 0.7;
extern bool   T3Original  = false;
extern bool   Interpolate = true;

//
//
//
//
//

double t3[];
double emat[][6];
double alpha;
double c1;
double c2;
double c3;
double c4;

//
//
//
//
//

int    timeFrame;
string IndicatorFileName;
bool   calculatingT3 = false;
bool   returningBars = false;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,t3);

   //
   //
   //
   //
   //
   
      if (TimeFrame=="calculateT3")
         {
            calculatingT3 = true;
         
            //
            //
            //
            //
            //
               
            double a  = T3Hot;
                   c1 = -a*a*a;
                   c2 = 3*a*a+3*a*a*a;
                   c3 = -6*a*a-3*a-3*a*a*a;
                   c4 = 1+3*a+a*a*a+3*a*a;

            T3Period = MathMax(1,T3Period);
            if (T3Original)
                 alpha = 2.0/(1.0 + T3Period);
            else alpha = 2.0/(2.0 + (T3Period-1.0)/2.0);
            return(0);
         }                   
      if (TimeFrame=="returnBars")
         {
            returningBars = true;
            return(0);
         }            

   //
   //
   //
   //
   //
               
   timeFrame = stringToTimeFrame(TimeFrame);
   IndicatorFileName = WindowExpertName();               
   IndicatorShortName("T3 "+TimeFrameToString(timeFrame)+"("+T3Period+")");
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=Bars-counted_bars;
         if (returningBars) { t3[0] = limit;      return(0); }
         if (calculatingT3) { CalculateT3(limit); return(0); }

   //
   //
   //
   //
   //
  
      if (timeFrame > Period()) limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));

      //
      //
      //
      //
      //
   
   	for(i = limit; i >= 0; i--)
      {
         int      shift1 = iBarShift(NULL,timeFrame,Time[i]);
         datetime time1  = iTime    (NULL,timeFrame,shift1);
   
            t3[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateT3",T3Period,T3Price,T3Hot,T3Original,0,shift1);
            if (timeFrame <= Period() || shift1==iBarShift(NULL,timeFrame,Time[i-1])) continue;
            if (!Interpolate) continue;		 

            //
            //
	         //
            //
            //

            for(int n = 1; i+n < Bars && Time[i+n] >= time1; n++) continue;	
            double factor = 1.0 / n;
            for(int k = 1; k < n; k++)
    	          t3[i+k] = k*factor*t3[i+n] + (1.0-k*factor)*t3[i];
      }
           

   //
   //
   //
   //
   //


   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void CalculateT3(int limit)
{
   for(int i=limit; i>=0; i--) t3[i] = iT3(emat,iMA(NULL,0,1,0,MODE_SMA,T3Price,i),i);
}

double iT3(double& emas[][6],double price,int shift)
{
   int i = Bars-shift-1;
   if (ArrayRange(emas,0) != Bars) ArrayResize(emas,Bars);
   if (i < 1)
      {
         emas[i][0] = price;
         emas[i][1] = price;
         emas[i][2] = price;
         emas[i][3] = price;
         emas[i][4] = price;
         emas[i][5] = price;
      }
   else
      {
         emas[i][0] = emas[i-1][0]+alpha*(price     -emas[i-1][0]);
         emas[i][1] = emas[i-1][1]+alpha*(emas[i][0]-emas[i-1][1]);
         emas[i][2] = emas[i-1][2]+alpha*(emas[i][1]-emas[i-1][2]);
         emas[i][3] = emas[i-1][3]+alpha*(emas[i][2]-emas[i-1][3]);
         emas[i][4] = emas[i-1][4]+alpha*(emas[i][3]-emas[i-1][4]);
         emas[i][5] = emas[i-1][5]+alpha*(emas[i][4]-emas[i-1][5]);
      }
   return(c1*emas[i][5] + c2*emas[i][4] + c3*emas[i][3] + c4*emas[i][2]);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   int tf=0;
       tfs = StringUpperCase(tfs);
         if (tfs=="M1" || tfs=="1")     tf=PERIOD_M1;
         if (tfs=="M5" || tfs=="5")     tf=PERIOD_M5;
         if (tfs=="M15"|| tfs=="15")    tf=PERIOD_M15;
         if (tfs=="M30"|| tfs=="30")    tf=PERIOD_M30;
         if (tfs=="H1" || tfs=="60")    tf=PERIOD_H1;
         if (tfs=="H4" || tfs=="240")   tf=PERIOD_H4;
         if (tfs=="D1" || tfs=="1440")  tf=PERIOD_D1;
         if (tfs=="W1" || tfs=="10080") tf=PERIOD_W1;
         if (tfs=="MN" || tfs=="43200") tf=PERIOD_MN1;
         if (tf<Period()) tf=Period();
  return(tf);
}
string TimeFrameToString(int tf)
{
   string tfs="";
   
   if (tf!=Period())
      switch(tf) {
         case PERIOD_M1:  tfs="M1"  ; break;
         case PERIOD_M5:  tfs="M5"  ; break;
         case PERIOD_M15: tfs="M15" ; break;
         case PERIOD_M30: tfs="M30" ; break;
         case PERIOD_H1:  tfs="H1"  ; break;
         case PERIOD_H4:  tfs="H4"  ; break;
         case PERIOD_D1:  tfs="D1"  ; break;
         case PERIOD_W1:  tfs="W1"  ; break;
         case PERIOD_MN1: tfs="MN1";
      }
   return(tfs);
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;
   int      lenght = StringLen(str) - 1;
   int      char;
   
   while(lenght >= 0)
      {
         char = StringGetChar(s, lenght);
         
         //
         //
         //
         //
         //
         
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                  s = StringSetChar(s, lenght, char - 32);
         else 
              if(char > -33 && char < 0)
                  s = StringSetChar(s, lenght, char + 224);
         lenght--;
   }
   
   //
   //
   //
   //
   //
   
   return(s);
}